package net.wanji.web.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.BeanMapUtils;
import net.wanji.databus.vo.SignalStatusVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.entity.TCrossControlHist;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.entity.TDeviceStatusLog;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.mapper.TDeviceStatusLogMapper;
import net.wanji.web.mapper.TDeviceStatusMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/03 10:43
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class SignalStatusTask {
    private final UtcFeignClients utcFeignClients;
    private final TDeviceStatusMapper tDeviceStatusMapper;
    private final ControlHistMapper controlHistMapper;
    private final TDeviceStatusLogMapper tDeviceStatusLogMapper;

    @Scheduled(fixedRate = 1 * 60 * 1000)
    @Transactional
    public void syncSignalStatus() {
        try {
            JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                log.error("定时任务同步信号机设备状态utcService调用异常");
                return;
            }
            List<Map<String, Object>> source = (List<Map<String, Object>>) jsonViewObject.getContent();
            List<SignalStatusVO> content = BeanMapUtils.mapsToObjects(source, SignalStatusVO.class);
            content.forEach(signalStatusVO -> {
                String signalId = signalStatusVO.getSignalId();
                String crossId = signalStatusVO.getCrossId();
                Integer currentSignalStatus = signalStatusVO.getStatus() == null ? 0 : signalStatusVO.getStatus();
                Integer currentFaultType = signalStatusVO.getFaultType() == null ? 0 : signalStatusVO.getFaultType();

                // 更新数据库状态
                LambdaQueryWrapper<TDeviceStatusInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                if (StringUtils.isNotEmpty(crossId)) {
                    lambdaQueryWrapper.eq(TDeviceStatusInfo::getCode, crossId);
                    TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(lambdaQueryWrapper);

                    if (isExucuteUpdate(currentSignalStatus, currentFaultType, tDeviceStatusInfo)) {
                        tDeviceStatusInfo.setStatus(currentSignalStatus);
                        tDeviceStatusInfo.setFaultType(currentFaultType);
                        tDeviceStatusInfo.setGmtCreate(new Date()); // 实时数据，创建和修改时间一致
                        tDeviceStatusMapper.updateById(tDeviceStatusInfo);
                        insertDeviceStatusLog(signalStatusVO, crossId, currentSignalStatus, currentFaultType);
                    }
                }
                insertControlHist(signalStatusVO);
            });
        } catch (Exception e) {
            log.error("定时任务同步信号机设备状态远程utcService调用异常", e);
        }
    }

    /**
     * 插入控制历史表
     *
     * @param signalStatusVO
     */
    private void insertControlHist(SignalStatusVO signalStatusVO) {
        // 更新控制历史表
        String crossId = signalStatusVO.getCrossId();
        Integer controlType = signalStatusVO.getControlType();
        TCrossControlHist tCrossControlHist = controlHistMapper.selectRecentOne(crossId);
        if (Objects.nonNull(tCrossControlHist) && !Objects.equals(controlType, tCrossControlHist.getType())) {
            controlHistMapper.insertOne(crossId, controlType, "未知");
        }
    }

    /**
     * 信号机状态插入设备日志表
     *
     * @param signalStatusVO
     * @param crossId
     * @param currentSignalStatus
     * @param currentFaultType
     */
    private void insertDeviceStatusLog(SignalStatusVO signalStatusVO, String crossId, Integer currentSignalStatus, Integer currentFaultType) {
        // 插入日志表
        TDeviceStatusLog tDeviceStatusLogInfoDB = tDeviceStatusLogMapper.selectLastOne(crossId);
        // DB数据是故障，如果当前正常非故障，状态为  FaultType=100 故障已处理
        if (Objects.nonNull(tDeviceStatusLogInfoDB) || tDeviceStatusLogInfoDB.getFaultType() != 0 && currentFaultType == 0) {
            tDeviceStatusLogInfoDB.setStatus(currentSignalStatus);
            tDeviceStatusLogInfoDB.setFaultType(100);
            tDeviceStatusLogInfoDB.setGmtCreate(new Date());
            tDeviceStatusLogInfoDB.setId(null);
            tDeviceStatusLogMapper.insert(tDeviceStatusLogInfoDB);
        } else {
            TDeviceStatusLog tDeviceStatusLog = new TDeviceStatusLog();
            tDeviceStatusLog.setCode(crossId);
            tDeviceStatusLog.setName(signalStatusVO.getSignalId());
            tDeviceStatusLog.setType(1);
            tDeviceStatusLog.setStatus(signalStatusVO.getStatus());
            tDeviceStatusLog.setFaultType(currentFaultType);
            tDeviceStatusLog.setGmtCreate(new Date());
            tDeviceStatusLog.setId(null);
            tDeviceStatusLogMapper.insert(tDeviceStatusLog);
        }
    }

    private static boolean isExucuteUpdate(Integer currentSignalStatus, Integer currentFaultType, TDeviceStatusInfo tDeviceStatusInfo) {
        return Objects.nonNull(tDeviceStatusInfo) && (currentSignalStatus != tDeviceStatusInfo.getStatus() || currentFaultType != tDeviceStatusInfo.getFaultType());
    }
}
